
#ifndef __NUC_INTERNAL__
#define __NUC_INTERNAL__

#include <string.h>
#include "NUC123Series.h"
#include "..\nubridge.h"
#include "NucBridgeApi.h"

// Flow Control Variable
extern volatile uint8_t g_u8IsCmd;

extern volatile uint8_t g_u8VComEnable;

extern volatile uint8_t timer0_tmp;

typedef struct {
    uint8_t type;
    uint8_t mode;
    uint16_t data1;
    uint16_t data2;
    uint16_t data3;
} _USB_CMD;

extern _USB_CMD g_tdsUsbCmd;

typedef struct s_mode_rec {
    uint8_t u8ParamSet[8];
    uint8_t u8ModeStatus[8];
} S_MODE_REC_T;

extern volatile uint8_t     g_u8SPI0Ready;
extern volatile uint8_t     g_u8SPI2Ready;
extern volatile uint8_t     g_u8I2C0Ready;
extern volatile uint8_t     g_u8PWMReady;
extern volatile uint8_t     g_u8ADCReady;

extern S_MODE_REC_T g_tdsModeInfo;

extern uint8_t g_u8CurrentModeSts;
extern uint8_t *g_u8ModeStatus;
extern uint8_t *g_u8ParamSet;

// Global Memory Allocation
#define RX_SIZE         0x4000
#define RX_BOUND        0x3FFF

#define TX_SIZE         0x400
#define ADCTX_SIZE      0x20

#define RX_SIZE_PART    0x800
#define RX_BOUND_PART   0x7FF

#define TX_SIZE_PART    0x400

#define CS_LABEL_SIZE   0x200
#define CS_LABEL_BOUND  0x1FF

extern uint8_t*             g_u8PDMARxBuf;
extern volatile uint8_t     g_u8PDMAValidByte;
extern volatile uint8_t     g_u8IsMaster;

extern uint8_t                      g_u8BulkOutBuf[64];

extern uint8_t                      g_u8RxBuf[RX_SIZE];
extern volatile uint16_t            g_u16RxTail;
extern volatile uint16_t            g_u16RxHead;
extern volatile uint16_t            g_u16RxByte;
extern volatile uint16_t            g_u16RxBufferOverflow;

extern uint8_t                      g_u8TxBuf[ADCTX_SIZE];//TX_SIZE
extern volatile uint16_t            g_u16TxTail;
extern volatile uint16_t            g_u16TxHead;
extern volatile uint16_t            g_u16TxByte;

// SPI0 Master
extern uint8_t*                     g_u8SPI0RxBuf;
extern volatile uint16_t            g_u16SPI0RxTail;
extern volatile uint16_t            g_u16SPI0RxHead;
extern volatile uint16_t            g_u16SPI0RxByte;

extern uint8_t*                     g_u8SPI0TxBuf;
extern volatile uint16_t            g_u16SPI0TxTail;
extern volatile uint16_t            g_u16SPI0TxHead;
extern volatile uint16_t            g_u16SPI0TxByte;

// SPI2 Master
extern uint8_t*                     g_u8SPI2RxBuf;
extern volatile uint16_t            g_u16SPI2RxTail;
extern volatile uint16_t            g_u16SPI2RxHead;
extern volatile uint16_t            g_u16SPI2RxByte;

extern uint8_t*                     g_u8SPI2TxBuf;
extern volatile uint16_t            g_u16SPI2TxTail;
extern volatile uint16_t            g_u16SPI2TxHead;
extern volatile uint16_t            g_u16SPI2TxByte;


extern uint8_t*                     g_u8I2C0RxBuf;
extern volatile uint16_t            g_u16I2C0RxTail;
extern volatile uint16_t            g_u16I2C0RxHead;
extern volatile uint16_t            g_u16I2C0RxByte;

extern uint8_t*                     g_u8I2C0TxBuf;
extern volatile uint16_t            g_u16I2C0TxTail;
extern volatile uint16_t            g_u16I2C0TxHead;
extern volatile uint16_t            g_u16I2C0TxByte;

extern uint16_t                     g_u16CS_LabelBuf[];
extern volatile uint16_t            g_u16CS_LabelTail;
extern volatile uint16_t            g_u16CS_LabelHead;
extern volatile uint16_t            g_u16CS_LabelByte;
extern volatile uint16_t            g_u16CS_OverFlow;

// PWM
extern uint8_t*                g_u8PWMRxBuf;//[PART_RX_SIZE];
extern volatile uint16_t       g_u16SPWMRxTail;
extern volatile uint16_t       g_u16PWMRxHead;
extern volatile uint16_t       g_u16PWMRxByte;

extern uint8_t*                g_u8PWMTxBuf;//[PART_TX_SIZE];
extern volatile uint16_t       g_u16PWMTxTail;
extern volatile uint16_t       g_u16PWMTxHead;
extern volatile uint16_t       g_u16PWMTxByte;

//Definitions for PWM
#define PWM_CAP_CHAN0   0x00
#define PWM_CAP_CHAN1   0x02
#define PWM_CAP_RISE    0x00
#define PWM_CAP_FALL    0x01
#define PWM_CAP_UNDERFLOW    0x04


extern uint8_t g_FindSlave;
extern uint8_t I2C_IndentifiedSlaveAddrBuf[];

// main.c
void UART0_DeInit(void);
void UART1_DeInit(void);



extern IRQn_Type IRQn_Rec;

void Nuc_Bridge_Init(void);
void Nuc_Bridge_Main(void);

#define GPIO_MONITOR1       1
#define GPIO_MONITOR2       2
#define SOFTWARE_I2C_MONITOR        GPIO_MONITOR2
void GPIO_Monitor_Init(void);
void GPIO_DeInit(void);

// SPI Master
void SPI0_Master_Init(void);
void SPI0_Master_DeInit(void);
void SPI0_DeInit_Param(SPI_T *port);
void SPI0_Master_MainProcess(void);

void SPI2_Master_Init(void);
void SPI2_Master_DeInit(void);
void SPI2_DeInit_Param(SPI_T *port);
void SPI2_Master_MainProcess(void);

void SPI_Slave_Init(void);
void SPI0_IRQHandler(void);
void SPI_Slave_DeInit(void);
void SPI_Slave_MainProcess(void);

void SPI0_Master_ReInit(uint8_t setting);
void SPI2_Master_ReInit(uint8_t setting);
void SPI0_Monitor_ReInit(uint8_t setting);
void I2C0_Master_ReInit(uint8_t setting);

//I2C Master Function Declaration
void I2C0_Master_Init(void);
void I2C0_Master_DeInit(void);
void I2C0_Master_MainProcess(void);

//I2C Monitor Function Declaration
void I2C_Monitor_DeInit(void);
void I2C_Monitor_Init(void);
void I2C_Monitor_MainProcess(void);

// VCOM
void UART0_Init(void);
void UART1_Init(void);

// PWM
void PWM_ReInit(uint8_t setting);
void PWM_Init(void);
void PWM_DeInit(void);
void PWM_MainProcess(void);

// ADC
void ADC_ReInit(uint8_t setting);
void ADC_Init(void);
void ADC_DeInit(void);
void ADC_MainProcess(void);

typedef struct {
//  uint8_t CLK_SRC;
//  uint8_t Divisor;
//  uint8_t EnabledChannels;
//  uint8_t OperationMode;
    uint8_t BurstSampleCnt;
    uint8_t TotalSampleCnt;
    uint8_t EnabledChannels;
} _ADC_INIT_T;

typedef struct {
    uint8_t  SampleCnt;
    uint32_t DataBuffer;
} _ADC_FILTER_T;

void Nuc_Reset(void);

extern volatile uint32_t        g_u32SPI0ApbClock;
extern volatile uint32_t        g_u32SPI2ApbClock;
extern volatile uint32_t        g_u32TEMP;

#endif  /* __NUC_INTERNAL__ */

