/**************************************************************************//**
 * @file     PWM_Function.c
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 14/01/22 11:27a $
 * @brief    NUC123 Series PWM Function Sample Code
 *
 * @note
 * Copyright (C) 2014 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/

/*!<Includes */
#include <stdio.h>
#include "NUC123Series.h"
#include "NucInternal.h"


/**
  * @brief  Initial PWM.
  * @param  None.
  * @retval None.
  */
void PWM_Init(void)
{
    PWM_ReInit(0x02);
}


/**
  * @brief  Initial PWM Function.
  * @param  setting:

  * @retval None.
  */
void PWM_ReInit(uint8_t setting)
{
    SYS_RESET_IP2(SYS_RST_PWM0123);
    SET_PWM01_CLKSRC(CLK_PWM01_HCLK_L, CLK_PWM01_HCLK_H);
    SET_PWM23_CLKSRC(CLK_PWM23_HCLK_L, CLK_PWM23_HCLK_H);

    SYS->GPA_MFP |= SYS_MFPA12_PWM0;
    SYS->GPA_MFP |= SYS_MFPA13_PWM1;
    SYS->GPC_MFP |= SYS_MFPC12_MISO11_PWM2 | SYS_MFPC13_MOSI11_PWM3;
    SYS->ALT_MFP |= SYS_ALT_PC12_I2SMCLK_PWM2 | SYS_ALT_PC13_CLKO_PWM3;

    APBCLK_ENABLE(CLK_PWM01 | CLK_PWM23) ;

    PWMA_POE_DISABLE(PWM_POE0);
    PWMA_CAP0_PA12_ENABLE();

    PWMA_POE_DISABLE(PWM_POE1);
    PWMA_CAP1_PA13_ENABLE();

    PWMA_POE_ENABLE(PWM_POE2) ;
    PWMA_POE_ENABLE(PWM_POE3) ;

    g_u8ParamSet[MODE_PWM] = setting;

    UNLOCKREG();
    NVIC_SetPriority(PWMA_IRQn,0);

}


/**
  * @brief  Disable PWM function
  * @param  None.
  * @retval None.
  */
void PWM_DeInit(void)
{
    //Set 16-bti down counter(CNR) as 0, and wait PDR to be 0
    PWMA->CNR0  =  0 ;
    PWMA->CNR1  =  0 ;
    PWMA->CNR2  =  0 ;
    PWMA->CNR3  =  0 ;
    while(PWMA->PDR0);
    while(PWMA->PDR1);
    while(PWMA->PDR2);
    while(PWMA->PDR3);

    DrvPWM_DisableInt(0);
    DrvPWM_DisableInt(1);
    DrvPWM_DisableInt(2);
    DrvPWM_DisableInt(3);
    PWM0_CAP_CLEAR_IF() ;
    PWM1_CAP_CLEAR_IF() ;
    PWMA_CLEAR_IF(PWM_IF2);
    PWMA_CLEAR_IF(PWM_IF3);

    SYS->GPA_MFP &= ~SYS_MFPA12_PWM0;
    SYS->GPA_MFP &= ~SYS_MFPA13_PWM1;

    SYS->GPC_MFP &= ~(SYS_MFPC12_MISO11_PWM2 | SYS_MFPC13_MOSI11_PWM3);
    SYS->ALT_MFP &= ~(SYS_ALT_PC12_I2SMCLK_PWM2 | SYS_ALT_PC13_CLKO_PWM3);

    DrvPWM_DeInit(0);
    DrvPWM_DeInit(1);
    DrvPWM_DeInit(2);
    DrvPWM_DeInit(3);
    g_u8ParamSet[MODE_PWM] = 0x00;
    g_u16RxBufferOverflow = 0;
}

/**
  * @brief  Parse the command script of g_u8PWMTxBuf[] and trigger the relative function.
  * @param  None.
  * @retval command remains not executed
  * @Details In this sample code, PWM2/3 are configured as PWM output and PWM0/1 are configured as PWM capture input.
  *
  * For example:
  * Byte 0: Total Command Count
  * (If PWM Output Function Enabled)                            (If PWM Capture Function Enabled)
  * Byte 1: Channel number(4:5)/Aligned mode(3)/divider(0:2)    1:Channel number(4:5)/Reserved(3)/divider(0:2)
  * Byte 2: CLK_SRC_H/CLK_SRC_L                                 2:CLK_SRC_H/CLK_SRC_L
  * Byte 3: Prescaler                                           3:Prescaler
  * Byte 4: CNR Low byte                                        4:CNR Low byte,0xFF recommended
  * Byte 5: CNR High byte                                       5:CNR High byte,0xFF recommended
  * Byte 6: CMR Low byte                                        6:CMR Low byte,0x00 recommended
  * Byte 7: CMR High byte                                       7:CMR High byte,0x00 recommended
  * Byte 8: DeadZone Enable/Mode                                8:FallingEnable(b'2)/RisingEnable(b'1)/Mode
  * Byte 9: DeadZone Interval
  */
uint8_t nChan;
S_DRVPWM_TIME_DATA_T  pPWM ;
uint8_t PWM_Run_Script(int *CmdCnt)
{



    while(*CmdCnt) {
        nChan = (g_u8PWMTxBuf[g_u16PWMTxHead]>>4)&0x03;
        pPWM.u8Div =  g_u8PWMTxBuf[g_u16PWMTxHead]&0x07;
        if(nChan > 1) {
            //Output mode
            if(g_u8PWMTxBuf[g_u16PWMTxHead]&0x08)
                PWMA_CH23_CENTER_ALIGN();
            else
                PWMA_CH23_EDGE_ALIGN();

            g_u16PWMTxHead++;

            SET_PWM23_CLKSRC((g_u8PWMTxBuf[g_u16PWMTxHead]&0xF)<<CLK_CLKSEL1_PWM23_S_Pos, ((g_u8PWMTxBuf[g_u16PWMTxHead]&0xF0)>>4)<<CLK_CLKSEL2_PWM23_S_EXT_Pos);
        } else {
            //Capture function
            if(g_u8PWMTxBuf[g_u16PWMTxHead]&0x08)
                PWMA_CH01_CENTER_ALIGN();
            else
                PWMA_CH01_EDGE_ALIGN();

            g_u16PWMTxHead++;

            SET_PWM01_CLKSRC((g_u8PWMTxBuf[g_u16PWMTxHead]&0xF)<<CLK_CLKSEL1_PWM01_S_Pos, ((g_u8PWMTxBuf[g_u16PWMTxHead]&0xF0)>>4)<<CLK_CLKSEL2_PWM01_S_EXT_Pos);
        }
        g_u16PWMTxHead++;
        pPWM.u8PreScale  =  g_u8PWMTxBuf[g_u16PWMTxHead++];
        pPWM.u16Cn       =  g_u8PWMTxBuf[g_u16PWMTxHead++] | (g_u8PWMTxBuf[g_u16PWMTxHead++]<<8);
        pPWM.u16Cm       =  g_u8PWMTxBuf[g_u16PWMTxHead++] | (g_u8PWMTxBuf[g_u16PWMTxHead++]<<8);
        pPWM.u8Mode      =  g_u8PWMTxBuf[g_u16PWMTxHead]&0x01;

        if(nChan < 2) {
            uint32_t IntMask = g_u8PWMTxBuf[g_u16PWMTxHead]&0x06 ;
            DrvPWM_CaptureEnable(nChan);
            DrvPWM_EnableCaptureInt(nChan,g_u8PWMTxBuf[g_u16PWMTxHead]&0x06);
            if(nChan & 0x1)  IntMask = (g_u8PWMTxBuf[g_u16PWMTxHead]&0x06) << 16 ;
            PWMA->CCR0 |= IntMask ;
            g_u16PWMTxHead++;
        } else if(g_u8PWMTxBuf[g_u16PWMTxHead]&0x02) {
            g_u16PWMTxHead++;
            DrvPWM_EnableDeadZone(nChan,g_u8PWMTxBuf[g_u16PWMTxHead++]);
        } else {
            g_u16PWMTxHead+=2;
        }

        pPWM.u8Inv       =  PWM_NO_INVERT;
        DrvPWM_SetClk(nChan, &pPWM) ;

        switch(nChan) {
        case 0 :
            PWMA->PCR   |= PWM_CH0_EN ;
            break;

        case 1 :
            PWMA->PCR   |= PWM_CH1_EN ;
            break ;

        case 2 :
            PWMA->PCR   |= PWM_CH2_EN ;
            break ;

        case 3 :
            PWMA->PCR   |= PWM_CH3_EN ;
            break ;
        }

        // parsing next command
        *CmdCnt = (*CmdCnt)-1;

        if(*CmdCnt == 0) {
            NVIC_EnableIRQ(PWMA_IRQn) ;
        }
    }

    return 0;
}



/*---------------------------------------------------------------------------------------------------------*/
/* MAIN function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/
void PWM_MainProcess(void)
{

    static int CmdCnt;

    if(g_u16PWMTxByte) {
        PWM_ReInit(0x12);

        CmdCnt = g_u8PWMTxBuf[g_u16PWMTxHead++];
        while(PWM_Run_Script(&CmdCnt)) {};

        g_u16PWMTxByte = 0;
        g_u16PWMTxHead = 0;

    }

}

/**
 * @brief       PWMA IRQHandler
 * @param       None
 * @return      None
 * @details     Every PWM1 down count to zero, execute this routines.
 */
void PWMA_IRQHandler()
{
    __IO static int32_t Duty_PWM0_FALL,Duty_PWM0_RISE,Duty_PWM1_FALL,Duty_PWM1_RISE;

    NVIC_DisableIRQ(PWMA_IRQn);
    if(PWM0_CAP_GET_IF()) {

        if(PWMA->CCR0 & PWM0_CAP_RISE_IE) {
            if(PWM0_CAP_GET_RISE()) {
                Duty_PWM0_RISE = PWM_CAP0_RISE_TIME();
                PWMA->CCR0 |= PWM0_CAP_RISE_FLAG;

//              g_u8RxBuf[g_u16RxTail++] = (PWMA_GET_IF(PWM_DIF0) >> 6) | PWM_CAP_CHAN0 | PWM_CAP_RISE;
                if(PWMA_GET_IF(PWM_DIF0)) {
                    PWMA_CLEAR_IF(PWM_DIF0);
                    g_u8RxBuf[g_u16RxTail++] = PWM_CAP_UNDERFLOW | PWM_CAP_CHAN0 | PWM_CAP_RISE;
                } else
                    g_u8RxBuf[g_u16RxTail++] = PWM_CAP_CHAN0 | PWM_CAP_RISE;

                g_u16RxTail &= RX_BOUND;
                g_u8RxBuf[g_u16RxTail++] = Duty_PWM0_RISE&0xFF;
                g_u16RxTail &= RX_BOUND;
                g_u8RxBuf[g_u16RxTail++] = (Duty_PWM0_RISE>>8)&0xFF;
                g_u16RxTail &= RX_BOUND;
                g_u16RxByte +=3;
            }
        }

        if(PWMA->CCR0 & PWM0_CAP_FALL_IE) {
            if(PWM0_CAP_GET_FALL()) {
                Duty_PWM0_FALL = PWM_CAP0_FALL_TIME();
                PWMA->CCR0 |= PWM0_CAP_FALL_FLAG;

//              g_u8RxBuf[g_u16RxTail++] = (PWMA_GET_IF(PWM_DIF0) >> 6) | PWM_CAP_CHAN0 | PWM_CAP_FALL;
                if(PWMA_GET_IF(PWM_DIF0)) {
                    PWMA_CLEAR_IF(PWM_DIF0);
                    g_u8RxBuf[g_u16RxTail++] = PWM_CAP_UNDERFLOW | PWM_CAP_CHAN0 | PWM_CAP_FALL;
                } else
                    g_u8RxBuf[g_u16RxTail++] = PWM_CAP_CHAN0 | PWM_CAP_FALL;

                g_u16RxTail &= RX_BOUND;
                g_u8RxBuf[g_u16RxTail++] = Duty_PWM0_FALL&0xFF;
                g_u16RxTail &= RX_BOUND;
                g_u8RxBuf[g_u16RxTail++] = (Duty_PWM0_FALL>>8)&0xFF;
                g_u16RxTail &= RX_BOUND;
                g_u16RxByte +=3;
            }
        }

        PWM0_CAP_CLEAR_IF();
        PWMA_CLEAR_IF(PWM_DIF0);
    }

    if(PWM1_CAP_GET_IF()) {
        if(PWMA->CCR0 & PWM1_CAP_FALL_IE) {
            if(PWM1_CAP_GET_FALL()) {
                Duty_PWM1_FALL = PWM_CAP1_FALL_TIME();
                PWMA->CCR0 |= PWM1_CAP_FALL_FLAG;

//                g_u8RxBuf[g_u16RxTail++] = (PWMA_GET_IF(PWM_DIF1) >> 7) | PWM_CAP_CHAN1 | PWM_CAP_FALL;
                if(PWMA_GET_IF(PWM_DIF1)) {
                    PWMA_CLEAR_IF(PWM_DIF1);
                    g_u8RxBuf[g_u16RxTail++] = PWM_CAP_UNDERFLOW | PWM_CAP_CHAN1 | PWM_CAP_FALL;
                } else
                    g_u8RxBuf[g_u16RxTail++] = PWM_CAP_CHAN1 | PWM_CAP_FALL;

                g_u16RxTail &= RX_BOUND;
                g_u8RxBuf[g_u16RxTail++] = Duty_PWM1_FALL&0xFF;
                g_u16RxTail &= RX_BOUND;
                g_u8RxBuf[g_u16RxTail++] = (Duty_PWM1_FALL>>8)&0xFF;
                g_u16RxTail &= RX_BOUND;
                g_u16RxByte +=3;
            }
        }

        if(PWMA->CCR0 & PWM1_CAP_RISE_IE) {
            if(PWM1_CAP_GET_RISE()) {
                Duty_PWM1_RISE = PWM_CAP1_RISE_TIME();
                PWMA->CCR0 |= PWM1_CAP_RISE_FLAG;

//                g_u8RxBuf[g_u16RxTail++] = (PWMA_GET_IF(PWM_DIF1) >> 7) | PWM_CAP_CHAN1 | PWM_CAP_RISE;
                if(PWMA_GET_IF(PWM_DIF1)) {
                    PWMA_CLEAR_IF(PWM_DIF1);
                    g_u8RxBuf[g_u16RxTail++] = PWM_CAP_UNDERFLOW | PWM_CAP_CHAN1 | PWM_CAP_RISE;
                } else
                    g_u8RxBuf[g_u16RxTail++] = PWM_CAP_CHAN1 | PWM_CAP_RISE;

                g_u16RxTail &= RX_BOUND;
                g_u8RxBuf[g_u16RxTail++] = Duty_PWM1_RISE&0xFF;
                g_u16RxTail &= RX_BOUND;
                g_u8RxBuf[g_u16RxTail++] = (Duty_PWM1_RISE>>8)&0xFF;
                g_u16RxTail &= RX_BOUND;
                g_u16RxByte +=3;
            }
        }

        PWM1_CAP_CLEAR_IF();

    }

    if(g_u16RxByte >=  RX_BOUND) {
        g_u16RxBufferOverflow = 1;
    } else {
        NVIC_EnableIRQ(PWMA_IRQn);
    }

}





