/******************************************************************************
 * @file     mouse.c
 * @brief    NANO100 series USBD driver source file
 * @version  2.0.0
 * @date     22, March, 2013
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "NUC123Series.h"
#include "sys.h"
#include "usbd.h"
#include "nubridge.h"

#include "..\NucFunction\NucInternal.h"

/*--------------------------------------------------------------------------*/
STR_VCOM_LINE_CODING gLineCoding = {115200, 0, 0, 8};   /* Baud rate : 115200    */
/* Stop bit     */
/* parity       */
/* data bits    */
uint16_t gCtrlSignal = 0;     /* BIT0: DTR(Data Terminal Ready) , BIT1: RTS(Request To Send) */

/*--------------------------------------------------------------------------*/
#define RXBUFSIZE           512 /* RX buffer size */
#define TXBUFSIZE           512 /* RX buffer size */

#define TX_FIFO_SIZE        16  /* TX Hardware FIFO size */


/*---------------------------------------------------------------------------------------------------------*/
/* Global variables                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
/* UART0 */
volatile uint8_t comRbuf[RXBUFSIZE];
volatile uint16_t comRbytes = 0;
volatile uint16_t comRhead = 0;
volatile uint16_t comRtail = 0;

volatile uint8_t comTbuf[TXBUFSIZE];
volatile uint16_t comTbytes = 0;
volatile uint16_t comThead = 0;
volatile uint16_t comTtail = 0;

uint8_t gRxBuf[64] = {0};
uint8_t *gpu8RxBuf = 0;
uint32_t gu32RxSize = 0;
uint32_t gu32TxSize = 0;

volatile int8_t gi8BulkOutReady = 0;
/********************************************************************/
uint32_t gu32InBuf[EP5_MAX_PKT_SIZE / 4] = {0};
uint32_t gu32InSize  = 0;
uint32_t gu32OutSize = 0;

/********************************************************************/
void SYS_Init(void)
{
    UNLOCKREG();                        //en: Unlock the locked registers
    GPIO_XT1();
    CLK_HXT_ENABLE();                                        //En:Enable External XTAL (4~24 MHz)
    while(!CLK_IS_STABLE(CLK_HXT_STB)); //En:Waiting for 12MHz clock ready
    SET_HCLK_CLKDIV(2);

    //en: configure PLL
    SET_PLL_CLKSRC(CLK_PLLSRC_HXT);     //en: Configure PLL Source is HXT
    /* PLL = 144MHz */
    CLK_SET_PLL_NO(CLK_PLLCON_NO_1);
    CLK_SET_PLL_NR(3);
    CLK_SET_PLL_NF(36);                     // 12 * 36/3 * 1/1 = 144MHz                     // 12 * 36/3 * 1/1 = 144MHz

    CLK_PLL_PD_DISABLE();
    CLK_PLL_OUTP_EN();
    while(!CLK_IS_STABLE(CLK_PLL_STB));     //en: Wait PLL Stable
    SET_HCLK_CLKDIV(2);
    SET_HCLK_CLKSRC(CLK_HCLK_PLL);          //en: Configure HCLK to be PLL output

    SystemCoreClockUpdate();                                //En:Update the variable SystemCoreClock

    /* Select USB divider source */
    SET_USB_CLKDIV(3);

    /* Enable USB device clock */
    APBCLK_ENABLE(CLK_USBD);

    LOCKREG(); //en: Re-lock the locked registers
}

void UART0_Init(void)
{
    GPIO_UART0_RXTX_PC();
    SYS_RESET_IP2(SYS_RST_UART0);                             //En:Reset UART0 IP
    APBCLK_ENABLE(CLK_UART0);                             //En:Enable UART0 clock
    SET_UART_CLKSRC(CLK_UART_HIRC);                        //En:Select internal 22MHz as UART clock source
    SET_UART_CLKDIV(1);                                      //En:Set UART clock divider number
    UART_SET_BAUD_MODE2(UART0, __IRC22M, 115200);               //En:Set UART0 Baudrate
    UART_SET_DATA_FORMAT(UART0, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_1); //En:Configure UART0 data format
    UART_ENABLE_INT(UART0, UART_IER_RDA_IEN|UART_IER_RLS_IEN);
    NVIC_EnableIRQ(UART0_IRQn);

}

void UART0_DeInit(void)
{
    NVIC_DisableIRQ(UART0_IRQn);
    outpw(&SYS->GPC_MFP, inpw(&SYS->GPC_MFP) & ~(0x3<<4));
    outpw(&SYS->ALT_MFP, inpw(&SYS->ALT_MFP) & ~(0x3<<29));
}

/*---------------------------------------------------------------------------------------------------------*/
/* UART Callback function                                                                                  */
/*---------------------------------------------------------------------------------------------------------*/
void UART0_IRQHandler(void)
{
    uint32_t u32IntStatus;
    uint8_t bInChar;
    int32_t size;

    u32IntStatus = UART0->ISR;

    if((u32IntStatus & 0x1 /* RDAIF */) || (u32IntStatus & 0x10 /* TOUT_IF */)) {
        /* Receiver FIFO threashold level is reached or Rx time out */

        /* Get all the input characters */
        while((UART0->FSR & UART_FSR_RX_EMPTY_Msk) == 0) {
            /* Get the character from UART Buffer */
            bInChar = UART0->DATA;

            /* Check if buffer full */
            if(comRbytes < RXBUFSIZE) {
                /* Enqueue the character */
                comRbuf[comRtail++] = bInChar;
                if(comRtail >= RXBUFSIZE)
                    comRtail = 0;
                comRbytes++;
            } else {
                /* FIFO over run */
            }
        }
    }

    if(u32IntStatus & 0x2 /* THRE_IF */) {

        if(comTbytes) {
            /* Fill the Tx FIFO */
            size = comTbytes;
            if(size >= TX_FIFO_SIZE) {
                size = TX_FIFO_SIZE;
            }

            while(size) {
                bInChar = comTbuf[comThead++];
                UART0->DATA = bInChar;
                if(comThead >= TXBUFSIZE)
                    comThead = 0;
                comTbytes--;
                size--;
            }
        } else {
            /* No more data, just stop Tx (Stop work) */
            UART0->IER &= (~UART_IER_THRE_IEN_Msk);
        }
    }

}

void UART1_Init(void)
{
    GPIO_UART1_RXTX();
    SYS_RESET_IP2(SYS_RST_UART1);                             //En:Reset UART1 IP
    APBCLK_ENABLE(CLK_UART1);                             //En:Enable UART1 clock
    SET_UART_CLKSRC(CLK_UART_HIRC);                        //En:Select internal 22MHz as UART clock source
    SET_UART_CLKDIV(1);                                      //En:Set UART clock divider number
    UART_SET_BAUD_MODE2(UART1, __IRC22M, 115200);               //En:Set UART0 Baudrate
    UART_SET_DATA_FORMAT(UART1, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_1); //En:Configure UART0 data format
    UART_ENABLE_INT(UART1, UART_IER_RDA_IEN|UART_IER_RLS_IEN);
    NVIC_EnableIRQ(UART1_IRQn);

}

void UART1_DeInit(void)
{
    NVIC_DisableIRQ(UART1_IRQn);
    outpw(&SYS->GPB_MFP, inpw(&SYS->GPC_MFP) & ~(0x3<<4));
    outpw(&SYS->ALT_MFP, inpw(&SYS->ALT_MFP) & ~(0x1<<15));
    outpw(&SYS->ALT_MFP, inpw(&SYS->ALT_MFP) & ~(0x1<<18));
}

void Timer0_Init(void)
{
    SET_TMR0_CLKSRC(CLK_TMR0_HCLK);
    APBCLK_ENABLE(CLK_TMR0)  ;      //Enable T0&T1 CLK

    ////Timer0 configure. Toggle PIN64:PB8_TM0 every 10ms ///////////////////////////////////////////
    DrvTIMER_Init(TIMER0, 255/*prescale*/, 10000/*TCMPR*/, TIM_MODE_PERIODIC);
    DrvTIMER_EnableInt(TIMER0, TIM_INT_EN);
}

void TMR0_IRQHandler(void)
{
    if(timer0_tmp & 0x01)
        GPIO_SET_PIN(PB14);
    else
        GPIO_CLR_PIN(PB14);

    timer0_tmp--;

    if(timer0_tmp == 0)
        NVIC_DisableIRQ(TMR0_IRQn);

    TIM_CLEAR_INTERRUPT_FLAG(TIMER0)  ;                           //Clear IF flag

}




/*---------------------------------------------------------------------------------------------------------*/
/* UART Callback function                                                                                  */
/*---------------------------------------------------------------------------------------------------------*/
void UART1_IRQHandler(void)
{
    uint32_t u32IntStatus;
    uint8_t bInChar;
    int32_t size;

    u32IntStatus = UART1->ISR;

    if((u32IntStatus & 0x1 /* RDAIF */) || (u32IntStatus & 0x10 /* TOUT_IF */)) {
        /* Receiver FIFO threashold level is reached or Rx time out */

        /* Get all the input characters */
        while((UART1->FSR & UART_FSR_RX_EMPTY_Msk) == 0) {
            /* Get the character from UART Buffer */
            bInChar = UART1->DATA;

            /* Check if buffer full */
            if(comRbytes < RXBUFSIZE) {
                /* Enqueue the character */
                comRbuf[comRtail++] = bInChar;
                if(comRtail >= RXBUFSIZE)
                    comRtail = 0;
                comRbytes++;
            } else {
                /* FIFO over run */
            }
        }
    }

    if(u32IntStatus & 0x2 /* THRE_IF */) {

        if(comTbytes) {
            /* Fill the Tx FIFO */
            size = comTbytes;
            if(size >= TX_FIFO_SIZE) {
                size = TX_FIFO_SIZE;
            }

            while(size) {
                bInChar = comTbuf[comThead++];
                UART1->DATA = bInChar;
                if(comThead >= TXBUFSIZE)
                    comThead = 0;
                comTbytes--;
                size--;
            }
        } else {
            /* No more data, just stop Tx (Stop work) */
            UART1->IER &= (~UART_IER_THRE_IEN_Msk);
        }
    }

}

/* Trasnfer data between USBD and UART */
void VCOM_TransferData(void)
{
    int32_t i, i32Len;

    /* Check if any data to send to USB & USB is ready to send them out */
    if(comRbytes && (gu32TxSize == 0)) {
        i32Len = comRbytes;
        if(i32Len > EP2_MAX_PKT_SIZE)
            i32Len = EP2_MAX_PKT_SIZE;

        for(i=0; i<i32Len; i++) {
            gRxBuf[i] = comRbuf[comRhead++];
            if(comRhead >= RXBUFSIZE)
                comRhead = 0;
        }

        NVIC_DisableIRQ(UARTV_IRQ);
        comRbytes -= i32Len;
        NVIC_EnableIRQ(UARTV_IRQ);

        gu32TxSize = i32Len;
        USBD_MemCopy((uint8_t *)(USBD_BUF_BASE + USBD_GET_EP_BUF_ADDR(EP2)), (uint8_t *)gRxBuf, i32Len);
        USBD_SET_PAYLOAD_LEN(EP2, i32Len);
    }

    /* Process the Bulk out data when bulk out data is ready. */
    if(gi8BulkOutReady && (gu32RxSize <= TXBUFSIZE - comTbytes)) {
        for(i=0; i<gu32RxSize; i++) {
            comTbuf[comTtail++] = gpu8RxBuf[i];
            if(comTtail >= TXBUFSIZE)
                comTtail = 0;
        }

        NVIC_DisableIRQ(UARTV_IRQ);
        comTbytes += gu32RxSize;
        NVIC_EnableIRQ(UARTV_IRQ);

        gu32RxSize = 0;
        gi8BulkOutReady = 0; /* Clear bulk out ready flag */

        /* Ready to get next BULK out */
        USBD_SET_PAYLOAD_LEN(EP3, EP3_MAX_PKT_SIZE);
    }

    /* Process the software Tx FIFO */
    if(comTbytes) {
        /* Check if Tx is working */
        if((UARTV->IER & UART_IER_THRE_IEN_Msk) == 0) {
            /* Send one bytes out */
            UARTV->DATA = comTbuf[comThead++];
            if(comThead >= TXBUFSIZE)
                comThead = 0;

            NVIC_DisableIRQ(UARTV_IRQ);
            comTbytes--;
            NVIC_EnableIRQ(UARTV_IRQ);

            /* Enable Tx Empty Interrupt. (Trigger first one) */
            UARTV->IER |= UART_IER_THRE_IEN_Msk;
        }
    }
}

void PrepareSendData(uint8_t *pData, uint32_t u32Length)
{
    gu32InSize = u32Length;
    USBD_MemCopy((uint8_t *)gu32InBuf, pData, u32Length);
}

void OnDataReceived(uint8_t *pData, uint32_t u32Length)
{
    uint8_t DataCnt = u32Length - 1;


    switch(pData[0]) {
    case 0:
        memcpy(g_u8BulkOutBuf, pData + 1, DataCnt);
        break;


    // SPI Master
    case MODE_SPI0_MASTER:
        if(g_u16SPI0TxByte == 0) {
            g_u16SPI0TxTail = 0;
            g_u16SPI0TxHead = 0;
        }
        memcpy(&(g_u8SPI0TxBuf[g_u16SPI0TxTail]), pData + 1, DataCnt);
        g_u16SPI0TxTail += DataCnt;
        g_u16SPI0TxByte += DataCnt;
        break;


    case MODE_SPI2_MASTER:
        if(g_u16SPI2TxByte == 0) {
            g_u16SPI2TxTail = 0;
            g_u16SPI2TxHead = 0;
        }
        memcpy(&(g_u8SPI2TxBuf[g_u16SPI2TxTail]), pData + 1, DataCnt);
        g_u16SPI2TxTail += DataCnt;
        g_u16SPI2TxByte += DataCnt;
        break;

    // I2C Master
    case MODE_I2C0_MASTER:
        if(g_u16I2C0TxByte == 0) {
            g_u16I2C0TxTail = 0;
            g_u16I2C0TxHead = 0;
        }
        memcpy(&(g_u8I2C0TxBuf[g_u16I2C0TxTail]), pData + 1, DataCnt);
        g_u16I2C0TxTail += DataCnt;
        g_u16I2C0TxByte += DataCnt;
        break;

    //PWM
    case MODE_PWM:
        if(g_u16PWMTxByte == 0) {
            g_u16PWMTxTail = 0;
            g_u16PWMTxHead = 0;
        }
        memcpy(&(g_u8PWMTxBuf[g_u16PWMTxTail]), pData + 1, DataCnt);
        g_u16PWMTxTail += DataCnt;
        g_u16PWMTxByte += DataCnt;
        break;

    //ADC
    case MODE_ADC:
        if(g_u16TxByte == 0) {
            g_u16TxTail = 0;
            g_u16TxHead = 0;
        }
        memcpy(&(g_u8TxBuf[g_u16TxTail]), pData + 1, DataCnt);
        g_u16TxTail += DataCnt;
        g_u16TxByte += DataCnt;
        break;
    }

}
void test_init_PWM(void);
void test_init_ADC(void);

/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main (void)
{
    SYS_Init();

//     UNLOCKREG();
//     BOD_ENABLE();
//     BOD_SET_VOLTAGE(SYS_BOD_27V);
//     LOCKREG();

    // Solved LD Rom GPIO Pin status
    SYS_RESET_IP2(SYS_RST_GPIO);


    // VCom Default Setting, 0: Disable, 1: Enable
    g_u8VComEnable = 1;

#if(VCOM_UART == 0)
    if(g_u8VComEnable)
        UART0_Init();
#else
    if(g_u8VComEnable)
        UART1_Init();
#endif

    //PB14 = output, LED
    GPIO_SET_MODE(PB,14,GPIO_OUTPUT) ;
    Timer0_Init();

    /* Initial USBD, set the descriptors and class request to driver */
    USBD_Open(&gsInfo, VCOM_ClassRequest, NULL);

    /* Endpoint configuration for CDC class (VCOM) */
    VCOM_Init();

    /* Enable USBD Interrupt */
    NVIC_EnableIRQ(USBD_IRQn);

    /* Start to transfer USB packet */
    USBD_Start();

    Nuc_Bridge_Init();

    /***************PWM ADC TEST START*******************/
//  PWM_Init();
//  test_init_PWM();
//  ADC_Init();
//  test_init_ADC();
    /***************PWM ADC TEST END*******************/

    while(1) {

        /***************PWM ADC TEST START*******************/
//      ADC_MainProcess();
//      PWM_MainProcess();
        /***************PWM ADC TEST END*******************/


        Nuc_Bridge_Main();

        if(g_u8VComEnable)
            VCOM_TransferData();
    }
}


//  * Byte 0: Total Command Count
//  * Byte 1: Number of channel(4:5)/Aligned mode(3)/divider(0:2)
//  * Byte 2: CLK_SRC_H/CLK_SRC_L
//  * Byte 3: Prescaler
//  * Byte 4: CNR Low byte
//  * Byte 5: CNR High byte
//  * Byte 6: CMR Low byte
//  * Byte 7: CMR High byte
//  * Byte 8: DeadZone Enable/Mode
//  * Byte 9: DeadZone Interval

void test_init_PWM(void)
{

    g_u16PWMTxTail = 0;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x03;//cmd cnt

    //PWM2 output
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x24;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x02;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x09;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x00;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x06;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x00;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x03;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x01;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x80;

    //PWM0 capture enable
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x04;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x02;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x09;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0xFF;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0xFF;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x01;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x00;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x05;

    //PWM1 capture enable
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x14;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x02;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x09;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0xFF;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0xFF;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x01;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x00;
    g_u8PWMTxBuf[g_u16PWMTxTail++] = 0x07;
    g_u16PWMTxByte = g_u16PWMTxTail;
}

void test_init_ADC(void)
{

    g_u8TxBuf[0] = 0x0C;//cmd cnt
    g_u8TxBuf[1] = 0x16;
    g_u8TxBuf[2] = 3;
    g_u8TxBuf[3] = 5;
    g_u8TxBuf[4] = 0xCF;

    g_u16TxTail += 4;
    g_u16TxByte += 4;
}



/*** (C) COPYRIGHT 2013 Nuvoton Technology Corp. ***/

